//  COMPILA  a UNIX, rscn i naos !!
// c++ -o tor6 tor6x5.cxx chrome1.cxx primitiv.cxx selector.cxx

//****************************************************************
//
//   tordl.cpp   dessenvolupa el problema sobre una
//                                           malla toroidal 
//   te quatre moviments:           3
//                                  |
//                              2-- 5-- 1
//                                  |
//                                  4
//   crea una grafica sobre el fitxer   GRAFICA
//   crea una evolucio grafica de la transmissio en el fitxer EVOL
//
//****************************************************************
//
//   Per evolucionar a partir de zero
//   evolucio = 1 (en aquest fitxer)   
//       i  en el fitxer CHROME.CPP
//   #define grafica 1/0  segons si es vol la grafica o no
//       #define mesh   0
//       #define toro   0
//       #define btfd   0
//       #define btfnd  0
//       #define ccc    0
//
//   Per introduir la expressio trobada
//   evolucio = 0     i en el fitxer CHROME:CPP
//   #define grafica 1/0 segons es vulgui trobar la grafica o no
//       #define mesh   0                                  
//       #define toro   1
//       #define btfd   0
//       #define btfnd  0
//       #define ccc    0
//		
//****************************************************************
//
// 


#include "pch.h"
#include "chrome.h"
#include "primitiv.h"

extern char scratch[100];
#ifdef FASTEVAL
        extern evalnode* IpGlobal;
        extern Chrome* ChromeGlobal;
#endif

//****************************************************************
// aqui es defineix la classe de problema

class GrilProb : public Problem {
public:
        GrilProb(ChromeParams* p);  // inicialitza funcions, 
                                    //parametres, i dades
        float fitness(Chrome* chrome);  // funcio GP fitness 
};

//****************************************************************
// variables globals que construeixen l'entorn d'evaluacio
// construeix aqui l'entorn del problema 

#define Horizontal 6
#define Vertical 5         // tamany de la malla toroidal
#define W 6                // nombre de pases
#define T 280              // increment positiu de la fitness
#define TPB 2000            // total de poblacio

int test[Horizontal][Vertical]; // prohibeix a un node tornar
// a actuar en un mateix pas
int Orden[Horizontal][Vertical];// pas de la sequencia de l'algorisme
int Paso[Horizontal][Vertical]; // etapa de pas de l'informacio
int Mesh[Horizontal][Vertical]; // paquets transmesos pel node
int Proc[Horizontal][Vertical]; // procedencia del paquet rebut
long double Vec_fitness[400];    // guarda la fitness per la grafica
long double Vec_mitja[400];      // guarda la mitja 
unsigned char graf[21][400];    // espai de la grafica
unsigned char M_evol[Horizontal][Vertical]; // guarda l'evolucio
// de la transmisio
int M_evol2[Horizontal][Vertical];
int pas,ord,Cambio,change,a1,a5,b1,move2,move1;
int expressio=0; // habilita l'evaluacio del tamany de l'expressio
int xx=0;        //posicio del node
int yy=0;
int h=0;         // comptador del tamany de l'expressio
int evolucio=1;  // per evolucionar a partir d'una expressio (1 evoluciona de 
                 // zero) (si val 0 ,l'expressio ha d'estar introduida a
		 // chrome.cxx i toro=1 a chrome.cpp)

typedef struct positions{  // serveix per poder guardar l'ordre en 
         int x;            // que s'escombren els nodes dintre de 
         int y;            // la xarxa
         } ;

positions position[Horizontal*Vertical]; 
// vector que conte aquestes posicions	 

//****************************************************************
//funcions que treballen amb les dades

 
retval ProcUp()   // evalua la procedencia del paquet rebut
{ 
        retval rval=0;	// si es un '4' indica procedencia
        rval= (Proc[xx][yy] == 4);    // per d'alt
        return rval;
}

retval ProcDown()   // evalua la procedencia del paquet rebut
{ 
        retval rval=0; // si es un '3' indica procedencia
        rval= (Proc[xx][yy] == 3);    // per baix
        return rval;
}

retval ProcLeft()   // evalua la procedencia del paquet rebut
{ 
        retval rval=0; // si es un '1' indica procedencia
        rval= (Proc[xx][yy] == 1);    // per l`esquerra
        return rval;
}

retval ProcRight()   // evalua la procedencia del paquet rebut
{ 
        retval rval=0; // si es un '2' indica procedencia
        rval= (Proc[xx][yy] == 2);    // per la dreta
        return rval;
}
	
void Resetest() // posa a zero la matriu 'test' per poder 
                //realitzar un altre pas
{              
        int i,j;
        for (j=0;j<Vertical;j++)
        {
          for (i=0;i<Horizontal;i++) test[i][j]=0;
        }
}							

void MouteUp()
{
        Proc[xx][yy-1]=3;          //  marca la procedencia
        position[ord].x=xx;
        position[ord].y=yy-1;
        Orden[xx][yy-1]=ord;ord++;
        Paso[xx][yy-1]=pas;        // indica el pas
        if (Cambio==1) yy=0;
        test[xx][yy]=1;            // prohibeix dintre del mateix
        // pas una nova transmissio
        Mesh[xx][yy]++;            // incrementa el comptador
        // de transmisions fetes pel node				 
}	

void MouteDown()
{
        Proc[xx][yy+1]=4;    
        position[ord].x=xx;
        position[ord].y=yy+1;		  // analogament
        Orden[xx][yy+1]=ord;ord++;
        Paso[xx][yy+1]=pas; 
        if (Cambio==1) yy=Vertical-1;
        test[xx][yy]=1;
        Mesh[xx][yy]++;
}	

void MouteRight()
{
        position[ord].x=xx+1;
        position[ord].y=yy;
        Orden[xx+1][yy]=ord;ord++;	   
        Proc[xx+1][yy]=1;
        Paso[xx+1][yy]=pas;
        if (Cambio==1) xx=Horizontal-1;
        Mesh[xx][yy]++;
        test[xx][yy]=1;	
}				 

void MouteLeft()
{
        position[ord].x=xx-1;
        position[ord].y=yy;
        Orden[xx-1][yy]=ord;ord++;	  	 
        Proc[xx-1][yy]=2;
        Paso[xx-1][yy]=pas;
        if (Cambio==1) xx=0;
        Mesh[xx][yy]++;
        test[xx][yy]=1;	 				 
}
			 
//****************************************************************
//////////////////////////  funcions especifiques del problema

// Us de OPDEF,EVAL,IP i CURCHROME definicions
// el codi es recompila per diferents metodes d'evaluacio
	  
OPDEF(IfProc_OriEval)       // mira si el node es l'origen
{ 
        retval rval=0;
        move1=0; move2=0;
        if (expressio==1) h++;  // comptador pel tamany 
                                // de l'expressio
        if (((xx==0)&&(yy==0)) || (expressio==1))
        {
          Proc[0][0]=5;     // marca amb un cinc l'origen
          move1=1;          // habilita la condicio per moure's
          rval=EVAL;        // evalua l'expresio
        }
        else {  
          IP++;
          TRAVERSE();    // salta l'expresio
          IP--;   		  // torna per evaluar
        }	
        return rval;
}

	class IfProc_Ori : public Function {
	public:
	IfProc_Ori(){strcpy(name,"IFPROC_ORI");argnum=1;
	varnum=0;weight=100;
	evalfunc=IfProc_OriEval;};
	};

OPDEF(IfProc_UpEval)    // mira si la procedencia es per dalt
{    
        retval rval=0;
        move1=0; move2=0;
        if (expressio==1) h++;
        if ((ProcUp()) || (expressio==1))
        {
          move1=1;
          rval=EVAL;				  
        }
        else {  
          IP++;
          TRAVERSE();
          IP--;							  
        }
        return rval;
}

	class IfProc_Up : public Function {
	public:
	IfProc_Up() {strcpy(name,"IFPROC_Up");argnum=1;
	varnum=0;weight=100;
	evalfunc=IfProc_UpEval;};
	};
							
OPDEF(IfProc_DownEval) // mira si la procedencia es inferior
{                      // i si ho es executa l'argument 
        retval rval=0;
        move1=0; move2=0;
        if (expressio==1) h++;
        if ((ProcDown()) || (expressio==1))
        {
          move1=1;
          rval=EVAL;				  
        }
        else {  
          IP++;
          TRAVERSE();
          IP--;							  
        }
        return rval;
}

	class IfProc_Down : public Function {
	public:
	IfProc_Down(){strcpy(name,"IFPROC_Down");argnum=1;
	varnum=0;weight=100;
	evalfunc=IfProc_DownEval;};
	};
						

OPDEF(IfProc_LeftEval) // mira si la procedencia es per l'esquerra
{    
        retval rval=0;
        move1=0; move2=0;
        if (expressio==1) h++;
        if ((ProcLeft()) || (expressio==1))
        {
          move1=1;
          rval=EVAL;				  
        }
        else {  
          IP++;
          TRAVERSE();
          IP--;							  
        }
        return rval;
}

	class IfProc_Left : public Function {
	public:
	IfProc_Left(){strcpy(name,"IFPROC_Left");argnum=1;
	varnum=0;weight=100;
	evalfunc=IfProc_LeftEval;};
	};
						
	  
OPDEF(IfProc_RightEval)// mira si la procedencia es per la dreta
{ 
        retval rval=0;
        move1=0; move2=0;
        if (expressio==1) h++;
        if ((ProcRight()) || (expressio==1))
        {
          move1=1;
          rval=EVAL;				  
        }
        else {  
          IP++;
          TRAVERSE();
          IP--;							  
        }
        return rval;
}

	class IfProc_Right : public Function {
	public:
	IfProc_Right(){strcpy(name,"IFPROC_Right");argnum=1;
	varnum=0;weight=100;
	evalfunc=IfProc_RightEval;};
	};


OPDEF(If_turn_0Eval)
{ 
			retval rval=0;
			move2=0;
			if (expressio==1) h++;
			if ((move1==1) || (expressio==1))
			{  
			  if ((Mesh[xx][yy]==0) || (expressio==1))
			  {  
			         move2=1;
			         rval=EVAL;
			  }
			  else {  
			         IP++;
			         TRAVERSE();
			         IP--;			  	
			  }
			}
			else rval=-1;	 		 
			return rval;
}

	class If_turn_0 : public Function {
	public:
	If_turn_0() {strcpy(name,"IF_TURN_0");argnum=1;
	varnum=0;weight=100;
	evalfunc=If_turn_0Eval;};
	};

OPDEF(If_turn_1Eval)
{ 
			retval rval=0; 
			move2=0;
			if (expressio==1) h++;
			if ((move1==1) || (expressio==1))
			{   
			  if ((Mesh[xx][yy]==1) || (expressio==1))
			  {  
			         move2=1;
			         rval=EVAL;
			  }
			  else {  
			         IP++;
			         TRAVERSE();
			         IP--;			  	
			  }
			}
			else rval=-1;	 		 
			return rval;
}

	class If_turn_1 : public Function {
	public:
	If_turn_1() {strcpy(name,"IF_TURN_1");argnum=1;
	varnum=0;weight=100;
	evalfunc=If_turn_1Eval;};
	};

OPDEF(If_turn_2Eval)
{ 
			retval rval=0; 
			move2=0;
			if (expressio==1) h++;
			if ((move1==1) || (expressio==1))
			{   
			  if ((Mesh[xx][yy]==2) || (expressio==1))
			  {  
			         move2=1;
			         rval=EVAL;
			  }
			  else {  
			         IP++;
			         TRAVERSE();
			         IP--;			  	
			  }
			}
			else rval=-1;	 		 
			return rval;
}

	class If_turn_2 : public Function {
	public:
	If_turn_2() {strcpy(name,"IF_TURN_2");argnum=1;
	varnum=0;weight=100;
	evalfunc=If_turn_2Eval;};
	};

OPDEF(If_turn_3Eval)
{ 
			retval rval=0; 
			move2=0;
			if (expressio==1) h++;
			if ((move1==1) || (expressio==1))
			{   
			  if ((Mesh[xx][yy]==3) || (expressio==1))
			  {  
			         move2=1;
			         rval=EVAL;
			  }
			  else {  
			         IP++;
			         TRAVERSE();
			         IP--;			  	
			  }
			}
			else rval=-1;	 		 
			return rval;
}

	class If_turn_3 : public Function {
	public:
	If_turn_3() {strcpy(name,"IF_TURN_3");argnum=1;
	varnum=0;weight=100;
	evalfunc=If_turn_3Eval;};
	};

OPDEF(MoveRight)  // si es pot moure per primer cop ho fa
                  // cap a la dreta
{
			retval rval=0;
			h++;
			if (expressio==0) 
			{ 
			  if (move2==1)
			  { 
			         if (test[xx][yy]==0) 
			         { 
			               if(xx==Horizontal-1){xx=-1;Cambio=1;}
			       // per fer-lo toroidal
			                if  (Proc[xx+1][yy]==0)// mira si el node de la
			                                       // dreta no ha rebut res
			                {
			                  MouteRight(); // transmet la informacio
			                  rval=T;   // incrementa la fitness positivament
			                            // un valor T
			                }
			                else  if (Cambio==1) xx=Horizontal-1; 
			         }	
			  }	
			  else rval=-1; // penalitza negativament transmetre sense mirar
			                // la procedencia
			}
			else h++;// comptador per penalitzar negativament la longitud de
			         // l'expressio
			Cambio=0; 
			return rval;
}	 	


OPDEF(MoveLeft)
{
			retval rval=0;
			h++;
			if (expressio==0) 
			{ 
			  if (move2==1)
			  { 
			         if (test[xx][yy]==0) 
			         { 
			                if (xx==0) {xx=Horizontal;Cambio=1;}
			                if  (Proc[xx-1][yy]==0)
			                {
			                  MouteLeft();
			                  rval=T;
			                }
			                else  if (Cambio==1) xx=0;
			         }		   
			  }	  
			  else rval=-1;
			}
			else h++; 
			Cambio=0;
			return rval;
}	 	

OPDEF(MoveUp)
{  
			retval rval=0;
			h++;
			if (expressio==0) 
			{
			  if (move2==1)
			  {   
			         if (test[xx][yy]==0) 
			         { 
			                if (yy==0){ yy=Vertical; Cambio=1;}
			                if (Proc[xx][yy-1]==0)
			                {		
			                  MouteUp();
			                  rval=T;
			                } 
			                else  if (Cambio==1) yy=0;
			         }		
			  }	
			  else  rval=-1; 
			}
			else h++;
			Cambio=0; 
			return rval;
}

OPDEF(MoveDown)
{  
			retval rval=0;
			h++;
			if (expressio==0)
			{
			  if (move2==1)
			  { 
			         if (test[xx][yy]==0)
			         {  
			                if (yy==Vertical-1) 
			                { yy=-1; Cambio=1;}
			                if (Proc[xx][yy+1]==0)
			                {
			                  MouteDown();
			                  rval=T;
			                }
			                else  if (Cambio==1) yy=Vertical-1; 
			         }
			  }	
			  else rval=-1; 
			}
			else h++;
			Cambio=0; 
			return rval;
}
  
OPDEF(Res) 
{
			if (expressio==1) h++;
			return 0;
}  


OPDEF(Progn1)  {return (EVAL);}

OPDEF(Progn2)  {return ( EVAL + EVAL);}

OPDEF(Progn3)  {return ( EVAL + EVAL + EVAL);}

OPDEF(Progn4)  {return ( EVAL + EVAL + EVAL + EVAL);}

OPDEF(Progn5)  {return ( EVAL + EVAL + EVAL + EVAL + EVAL);}

OPDEF(Progn6)  {return ( EVAL + EVAL + EVAL + EVAL + EVAL + EVAL);}

//****************************************************************
///////////////////////// definicio del PROBLEMA

GrilProb::GrilProb(ChromeParams* p) : Problem()
{
	AddF(new ConstFunc(0)); // Requerida per GPQUICK, pero pes zero

	// Problem specific functions
	AddF(new Function(1,0,100,IfProc_UpEval,"IfProc_Up"));
	AddF(new Function(1,0,100,IfProc_DownEval,"IfProc_Down"));
	AddF(new Function(1,0,100,IfProc_LeftEval,"IfProc_Left"));
	AddF(new Function(1,0,100,IfProc_RightEval,"IfProc_Right"));
	AddF(new Function(1,0,150,IfProc_OriEval,"IfProc_Ori"));
 
	AddF(new Function(1,0,100,If_turn_0Eval,"If_turn_0"));     
	AddF(new Function(1,0,100,If_turn_1Eval,"If_turn_1"));
	AddF(new Function(1,0,100,If_turn_2Eval,"If_turn_2"));		  
	AddF(new Function(1,0,100,If_turn_3Eval,"If_turn_3"));
	AddF(new Function(0,0,100,MoveLeft,"MoveLeft"));
	AddF(new Function(0,0,100,MoveRight,"MoveRight"));
	AddF(new Function(0,0,100,MoveUp,"MoveUp"));
	AddF(new Function(0,0,100,MoveDown,"MoveDown"));
	 
	AddF(new Function(6,0,150,Progn6,"Prog6")); 
	AddF(new Function(5,0,150,Progn5,"Prog5")); 
	AddF(new Function(4,0,125,Progn4,"Prog4"));
	AddF(new Function(3,0,125,Progn3,"Prog3"));
	AddF(new Function(2,0,100,Progn2,"Prog2"));
	AddF(new Function(1,0,10,Progn1,"Prog1")); 

	AddF(new Function(0,0,75,Res,"Res"));
	// Set some parameters
	p->params[pRepeatEval] = 0;       // mateix cas tot el temps
	p->params[pMuteConstWt] = 0;      // sense constants
	 // Get to Koza type parameters
	p->params[pMuteWt] = 15;          // baixa mutacio
	p->params[pAnnealMuteWt] = 15;    // desactiva annealing
	p->params[pAnnealCrossWt] = 0;
	p->params[pCrossSelf] = 0;        // possibilita creuament amb si
                                          // mateix
	p->params[pMaxAge] = 500;
	p->params[pInitExpr] = 20;
	p->params[pMaxExpr] = 90;
	p->params[pMuteRate] = 100;

	
	// si es necessita introduir dades especifiques
	// del problema per l'evaluacio
	// o introduir un entorn (un mapa, un escenari)
	// fer-ho aqui 
}

float GrilProb::fitness(Chrome* chrome)    // calcula
      // la fitness aplicant l`expressio
{                           // sobre la xarxa pas a pas
	int i,j,s;
	float f=0;
	for (i=0;i<Vertical;i++)  // posa totes les matrius a zero
	  for (j=0;j<Horizontal;j++)
	  {
	       Proc[j][i]=0;Mesh[j][i]=0;Paso[j][i]=0;
	       Orden[j][i]=0;test[j][i]=0;
	  }
	for (i=0;i<Horizontal*Vertical;i++)
	{
	  position[i].x=0;
	  position[i].y=0;
	}            
	xx=0;yy=0; // inicialitzacio de tos els valors a zero
	h=0;
	ord=1; pas=1;
	retval answer=0;
	retval s1=0;
	// mecanisme d'escombrat de la xarxa	  
	for (s=0;s<W;s++)     // bucle pels pasos
	{ 
	  for (i=0;i<Horizontal*Vertical;i++) 
	  {
	         xx=position[i].x;
	         yy=position[i].y;
	         if ( (i==0) || ((Proc[xx][yy] != 0) && (Mesh[xx][yy]<4)
	             && (Paso[xx][yy]<pas) && (test[xx][yy]==0)))
	         {       // executa tot aixo si el node te 
		         //informacio per transmetre, ha 
	                 // transmes menys de 4 cops i no ha rebut 
	                 //informacio en aquest pas	
	                move2=0;move1=0;   // evalua l'expressio
	                answer+= chrome->evalAll();   // sencera
	                f=answer; 
	                h=0;
	                xx=0;yy=0;
	                expressio=1; // evalua la seva longitud per
	                s1+= chrome->evalAll();     // minimitzarla
	                expressio=0;
	                f=f-(h*2); // li resta a la fitness
	                chrome->nfitness->fvalue=f;			 
		 }
	  }		
	  if (ord<=(Horizontal*Vertical)-1)
	  {
	         pas++;
	         Resetest();
	  }
	}
	return f;
}

//****************************************************************
///////////////////////////////  MAIN

int EvolveAnt()
// funciona amb una poblacio de tamany TPB
// funciona per 89,000 evaluacions o fins que es premi una tecla
// cada 5 seconds o 1000 evaluacions, presenta el millor
{
	ChromeParams* params = new ChromeParams();
	Problem* prob=new GrilProb(params);
	Pop* pop;

	cout << "\nGPQUICK Artificial TORI MESH Problem\n";

	srand(1);// en lloc de rndomize();que causava problemes al compilar
	pop = new Pop(prob,params,TPB);  // poblacio de tamany TPB
	a1=0;b1=0;
	int i,j,a3,a6,k;
	long double a2;
	ofstream fin("TOR6x5.GRF",ios::out | ios::app);
	fin<<"\n";
	for (i=0;i<80;i++) fin<<'=';
	fin<<"\nMesh no dirigit ->"<<Horizontal<<"x"
	<<Vertical<<" en "<<W<<" pasos";
	fin<<"\nNombre de Poblacio "<<TPB<<"\n";
	
	//while (pop->gencount<39000l && !done)  // versio- no interrumpible
	while (pop->gencount<89000l)  // kbhit Is NOT PORTABLE (Borland specific)
	{   
	        // go cada 5 seconds o 1000 evaluacions
	        pop->go_until(time(NULL)+100,3000);
				
	        // presenta el millor
	        cout << "\niteracions: " << pop->gencount << 
			"\t valor ponderat: " << pop->BestFitness->fvalue << 
			"\t valor mitja: " << Vec_mitja[a1] << 
			"\t millor:	" << pop->BestMember <<
			"\n";
	        pop->pop[pop->BestMember]->write(PRETTY_NONE,cout);
	        cout.flush();
	        a1++;	
			fin<< "\n"<<pop->gencount;   // iteracions fetes	        
	        fin<< "\t"<<Vec_mitja[a1];   // mitjana
			fin<< "\t"<<pop->BestFitness->fvalue;  //millor	
	}		
 	// presenta el millor

	ofstream of("TOR6x5.RES",ios::out | ios::app);
	cout << "\n\nFINAL RESULTS...";
	of<<"\n\nMesh no dirigit de "<<Horizontal<<"x"<<Vertical
	<<" en "<<W<<" pasos, T="<<T<<" i TPB="<<TPB;
	cout << "\nAfter " << pop->gencount << " generates, 
	#fitness found = " << pop->BestFitness->fvalue << "\n";
	pop->pop[pop->BestMember]->write(PRETTY_NONE,cout);
	cout.flush();
	cout<<"****";pop->pop[pop->BestMember]->SetupEval();
	float f=0;
		
		 // presenta el millor. no resta per la longitud
		 // perque ja no es necessari.			
	xx=0; yy=0;h=0;
	ord=1;pas=1;expressio=0;
	for (i=0;i<Vertical;i++)
	  for (j=0;j<Horizontal;j++)
	  {
	    Proc[j][i]=0;Mesh[j][i]=0;Paso[j][i]=0;
	    Orden[j][i]=0;test[j][i]=0;
	  }
	for (i=0;i<Horizontal*Vertical;i++)
	{
	  position[i].x=0;
	  position[i].y=0;
	}            
	retval answer=0; 
		  
			 for (int s=0;s<W;s++)        // bucle pels pasos
			 { 
			        for (i=0;i<Horizontal*Vertical;i++) 
			        { 
			          xx=position[i].x;
			          yy=position[i].y;
			          if ( (i==0) || ((Proc[xx][yy] != 0) &&
 			              (Mesh[xx][yy]<4) && (Paso[xx][yy]<pas) 
			               && (test[xx][yy]==0)))
			          {
					  // evalua la millor expressio
			                 move2=0;h=0;move1=0; 
			                 answer+= pop->pop[pop->BestMember]->evalAll();
			                 f=answer;
			          }											
			        }
			        if (ord<=(Horizontal*Vertical)-1)
			        {
			          pas++;
			          Resetest();
			        }
		          }		
		 // mostra el resultat
	f=answer;
	cout << '\n';
	of << '\n';
	for (int y=0; y<Vertical; y++)
	{  
	  fin<<"\n";
	  for (int x=0; x<Horizontal; x++)
	  {
	         cout<<Proc[x][y]<< "\t" << Paso[x][y]<<"\t ";
	         of<<Proc[x][y]<< "\t" << Paso[x][y] <<"\t ";
	         fin<<Proc[x][y]<< "\t" << Paso[x][y] <<"\t ";
	  }
	  cout<<"\n";
	  of<<"\n";			 
	}  
					
	cout<<"\nFitness="<<f << "After " << pop->gencount<< " generates";
	of<<"\nFitness="<<f << " After " << pop->gencount<< " generates";
	fin<<"\nFitness="<<f << " After " << pop->gencount<< " generates";
	cout<<"\n";pop->pop[pop->BestMember]->write(PRETTY_NONE,cout);
	of<<"\n";pop->pop[pop->BestMember]->write(PRETTY_NONE,of);
	fin<<"\n";pop->pop[pop->BestMember]->write(PRETTY_NONE,fin);
	cout << "\nResult in TOR6x5.RES";

			 // Pinta l`evolucio en la transmisio
	ofstream fa1("TOR6x5.EVL",ios::out | ios::app);
	fa1<<"\n FITXER D'EVOLUCIO DE LA TRANSMISIO";
	for(i=0;i<Horizontal;i++)
	  for(j=0;j<Vertical;j++) M_evol[i][j]=' ';	
	for(i=0;i<=W;i++)
	{
	  for(j=0;j<Horizontal;j++)
	     for(k=0;k<Vertical;k++)
	        if (Paso[j][k]==i)
	        { 
	            M_evol[j][k]='*';
	            M_evol2[j][k]=i;
	        }  
	  fa1<<"\n paso  "<<i<<hex;
	  for(k=0;k<Vertical;k++)
	  {
                 fa1<<"\n"<<"\t";
       	         for(j=0;j<Horizontal;j++)
	           if (M_evol2[j][k]!='.') 
	             fa1<<M_evol2[j][k]<<hex;		
	           else fa1<<M_evol[j][k];
	  }
	  fa1<<"\n"<<"\t";
	  for(j=0;j<Horizontal;j++) fa1<<'^';	 
	  for(j=0;j<Horizontal;j++)
	         for(k=0;k<Vertical;k++)
	                if (M_evol[j][k]=='*')
	                {
	                  M_evol[j][k]='.';
    	                  M_evol2[j][k]='.';
   	                }
	}
	fa1.close();
	of.close();	 
	fin.close();

	delete pop;
	delete prob;
	delete params;
	return 0;
}

int main(int, char **)
{
        EvolveAnt();
        return 0;
}
